@@ -1038,10 +1038,10 @@ CheckValidResultRel(Relation resultRel, CmdType operation)
 
 			/*
 			 * Okay only if there's a suitable INSTEAD OF trigger.  Messages
-			 * here should match rewriteHandler.c's rewriteTargetView and
-			 * RewriteQuery, except that we omit errdetail because we haven't
-			 * got the information handy (and given that we really shouldn't
-			 * get here anyway, it's not worth great exertion to get).
+			 * here should match rewriteHandler.c's rewriteTargetView, except
+			 * that we omit errdetail because we haven't got the information
+			 * handy (and given that we really shouldn't get here anyway, it's
+			 * not worth great exertion to get).
 			 */
 			switch (operation)
 			{
@@ -2289,13 +2289,7 @@ EvalPlanQualFetch(EState *estate, Relation relation, int lockmode,
 
 			/* otherwise xmin should not be dirty... */
 			if (TransactionIdIsValid(SnapshotDirty.xmin))
-				ereport(ERROR,
-						(errcode(ERRCODE_DATA_CORRUPTED),
-						 errmsg_internal("t_xmin %u is uncommitted in tuple (%u,%u) to be updated in table \"%s\"",
-										 SnapshotDirty.xmin,
-										 ItemPointerGetBlockNumber(&tuple.t_self),
-										 ItemPointerGetOffsetNumber(&tuple.t_self),
-										 RelationGetRelationName(relation))));
+				elog(ERROR, "t_xmin is uncommitted in tuple to be updated");
 
 			/*
 			 * If tuple is being updated by other transaction then we have to
