@@ -621,7 +621,7 @@ standard_ProcessUtility(Node *parsetree,
 			{
 				UnlistenStmt *stmt = (UnlistenStmt *) parsetree;
 
-				/* we allow UNLISTEN during recovery, as it's a noop */
+				PreventCommandDuringRecovery("UNLISTEN");
 				CheckRestrictedOperation("UNLISTEN");
 				if (stmt->conditionname)
 					Async_Unlisten(stmt->conditionname);
@@ -963,34 +963,24 @@ ProcessUtilitySlow(Node *parsetree,
 			case T_CreateForeignTableStmt:
 				{
 					List	   *stmts;
-					RangeVar   *table_rv = NULL;
+					ListCell   *l;
 
 					/* Run parse analysis ... */
 					stmts = transformCreateStmt((CreateStmt *) parsetree,
 												queryString);
 
-					/*
-					 * ... and do it.  We can't use foreach() because we may
-					 * modify the list midway through, so pick off the
-					 * elements one at a time, the hard way.
-					 */
-					while (stmts != NIL)
+					/* ... and do it */
+					foreach(l, stmts)
 					{
-						Node	   *stmt = (Node *) linitial(stmts);
-
-						stmts = list_delete_first(stmts);
+						Node	   *stmt = (Node *) lfirst(l);
 
 						if (IsA(stmt, CreateStmt))
 						{
-							CreateStmt *cstmt = (CreateStmt *) stmt;
 							Datum		toast_options;
 							static char *validnsps[] = HEAP_RELOPT_NAMESPACES;
 
-							/* Remember transformed RangeVar for LIKE */
-							table_rv = cstmt->relation;
-
 							/* Create the table itself */
-							address = DefineRelation(cstmt,
+							address = DefineRelation((CreateStmt *) stmt,
 													 RELKIND_RELATION,
 													 InvalidOid, NULL);
 							EventTriggerCollectSimpleCommand(address,
@@ -1008,7 +998,7 @@ ProcessUtilitySlow(Node *parsetree,
 							 * table
 							 */
 							toast_options = transformRelOptions((Datum) 0,
-																cstmt->options,
+											  ((CreateStmt *) stmt)->options,
 																"toast",
 																validnsps,
 																true,
@@ -1022,37 +1012,16 @@ ProcessUtilitySlow(Node *parsetree,
 						}
 						else if (IsA(stmt, CreateForeignTableStmt))
 						{
-							CreateForeignTableStmt *cstmt = (CreateForeignTableStmt *) stmt;
-
-							/* Remember transformed RangeVar for LIKE */
-							table_rv = cstmt->base.relation;
-
 							/* Create the table itself */
-							address = DefineRelation(&cstmt->base,
+							address = DefineRelation((CreateStmt *) stmt,
 													 RELKIND_FOREIGN_TABLE,
 													 InvalidOid, NULL);
-							CreateForeignTable(cstmt,
+							CreateForeignTable((CreateForeignTableStmt *) stmt,
 											   address.objectId);
 							EventTriggerCollectSimpleCommand(address,
 															 secondaryObject,
 															 stmt);
 						}
-						else if (IsA(stmt, TableLikeClause))
-						{
-							/*
-							 * Do delayed processing of LIKE options.  This
-							 * will result in additional sub-statements for us
-							 * to process.  Those should get done before any
-							 * remaining actions, so prepend them to "stmts".
-							 */
-							TableLikeClause *like = (TableLikeClause *) stmt;
-							List	   *morestmts;
-
-							Assert(table_rv != NULL);
-
-							morestmts = expandTableLikeClause(table_rv, like);
-							stmts = list_concat(morestmts, stmts);
-						}
 						else
 						{
 							/*
@@ -1069,7 +1038,7 @@ ProcessUtilitySlow(Node *parsetree,
 						}
 
 						/* Need CCI between commands */
-						if (stmts != NIL)
+						if (lnext(l) != NULL)
 							CommandCounterIncrement();
 					}
 
@@ -1278,7 +1247,6 @@ ProcessUtilitySlow(Node *parsetree,
 					IndexStmt  *stmt = (IndexStmt *) parsetree;
 					Oid			relid;
 					LOCKMODE	lockmode;
-					bool		is_alter_table;
 
 					if (stmt->concurrent)
 						PreventTransactionChain(isTopLevel,
@@ -1301,17 +1269,6 @@ ProcessUtilitySlow(Node *parsetree,
 												 RangeVarCallbackOwnsRelation,
 												 NULL);
 
-					/*
-					 * If the IndexStmt is already transformed, it must have
-					 * come from generateClonedIndexStmt, which in current
-					 * usage means it came from expandTableLikeClause rather
-					 * than from original parse analysis.  And that means we
-					 * must treat it like ALTER TABLE ADD INDEX, not CREATE.
-					 * (This is a bit grotty, but currently it doesn't seem
-					 * worth adding a separate bool field for the purpose.)
-					 */
-					is_alter_table = stmt->transformed;
-
 					/* Run parse analysis ... */
 					stmt = transformIndexStmt(relid, stmt, queryString);
 
@@ -1321,7 +1278,7 @@ ProcessUtilitySlow(Node *parsetree,
 						DefineIndex(relid,		/* OID of heap relation */
 									stmt,
 									InvalidOid, /* no predefined OID */
-									is_alter_table,
+									false,		/* is_alter_table */
 									true,		/* check_rights */
 									false,		/* skip_build */
 									false);		/* quiet */
