@@ -28,8 +28,6 @@
 #include "fe_utils/mbprint.h"
 
 
-#define PQmblenBounded(s, e)  strnlen(s, PQmblen(s, e))
-
 static bool ExecQueryUsingCursor(const char *query, double *elapsed_msec);
 static bool command_no_begin(const char *query);
 static bool is_select_command(const char *query);
@@ -380,27 +378,13 @@ CheckConnection(void)
 		if (!OK)
 		{
 			psql_error("Failed.\n");
-
-			/*
-			 * Transition to having no connection.  Keep this bit in sync with
-			 * do_connect().
-			 */
 			PQfinish(pset.db);
 			pset.db = NULL;
 			ResetCancelConn();
 			UnsyncVariables();
 		}
 		else
-		{
 			psql_error("Succeeded.\n");
-
-			/*
-			 * Re-sync, just in case anything changed.  Keep this in sync with
-			 * do_connect().
-			 */
-			SyncVariables();
-			connection_warnings(false); /* Must be after SyncVariables */
-		}
 	}
 
 	return OK;
@@ -797,13 +781,6 @@ StoreQueryTuple(const PGresult *result)
 			/* concatenate prefix and column name */
 			varname = psprintf("%s%s", pset.gset_prefix, colname);
 
-			if (VariableHasHook(pset.vars, varname))
-			{
-				psql_error("attempt to \\gset into specially treated variable \"%s\" ignored\n",
-						   varname);
-				continue;
-			}
-
 			if (!PQgetisnull(result, 0, i))
 				value = PQgetvalue(result, 0, i);
 			else
@@ -969,49 +946,20 @@ ProcessResult(PGresult **results)
 			 * connection out of its COPY state, then call PQresultStatus()
 			 * once and report any error.
 			 *
-			 * For COPY OUT, direct the output to pset.copyStream if it's set,
-			 * otherwise to pset.gfname if it's set, otherwise to queryFout.
-			 * For COPY IN, use pset.copyStream as data source if it's set,
-			 * otherwise cur_cmd_source.
+			 * If pset.copyStream is set, use that as data source/sink,
+			 * otherwise use queryFout or cur_cmd_source as appropriate.
 			 */
-			FILE	   *copystream;
+			FILE	   *copystream = pset.copyStream;
 			PGresult   *copy_result;
 
 			SetCancelConn();
 			if (result_status == PGRES_COPY_OUT)
 			{
-				bool		need_close = false;
-				bool		is_pipe = false;
-
-				if (pset.copyStream)
-				{
-					/* invoked by \copy */
-					copystream = pset.copyStream;
-				}
-				else if (pset.gfname)
-				{
-					/* invoked by \g */
-					if (openQueryOutputFile(pset.gfname,
-											&copystream, &is_pipe))
-					{
-						need_close = true;
-						if (is_pipe)
-							disable_sigpipe_trap();
-					}
-					else
-						copystream = NULL;	/* discard COPY data entirely */
-				}
-				else
-				{
-					/* fall back to the generic query output stream */
+				if (!copystream)
 					copystream = pset.queryFout;
-				}
-
 				success = handleCopyOut(pset.db,
 										copystream,
-										&copy_result)
-					&& success
-					&& (copystream != NULL);
+										&copy_result) && success;
 
 				/*
 				 * Suppress status printing if the report would go to the same
@@ -1023,25 +971,11 @@ ProcessResult(PGresult **results)
 					PQclear(copy_result);
 					copy_result = NULL;
 				}
-
-				if (need_close)
-				{
-					/* close \g argument file/pipe */
-					if (is_pipe)
-					{
-						pclose(copystream);
-						restore_sigpipe_trap();
-					}
-					else
-					{
-						fclose(copystream);
-					}
-				}
 			}
 			else
 			{
-				/* COPY IN */
-				copystream = pset.copyStream ? pset.copyStream : pset.cur_cmd_source;
+				if (!copystream)
+					copystream = pset.cur_cmd_source;
 				success = handleCopyIn(pset.db,
 									   copystream,
 									   PQbinaryTuples(*results),
@@ -1702,7 +1636,7 @@ skip_white_space(const char *query)
 
 	while (*query)
 	{
-		int			mblen = PQmblenBounded(query, pset.encoding);
+		int			mblen = PQmblen(query, pset.encoding);
 
 		/*
 		 * Note: we assume the encoding is a superset of ASCII, so that for
@@ -1739,7 +1673,7 @@ skip_white_space(const char *query)
 					query++;
 					break;
 				}
-				query += PQmblenBounded(query, pset.encoding);
+				query += PQmblen(query, pset.encoding);
 			}
 		}
 		else if (cnestlevel > 0)
@@ -1774,7 +1708,7 @@ command_no_begin(const char *query)
 	 */
 	wordlen = 0;
 	while (isalpha((unsigned char) query[wordlen]))
-		wordlen += PQmblenBounded(&query[wordlen], pset.encoding);
+		wordlen += PQmblen(&query[wordlen], pset.encoding);
 
 	/*
 	 * Transaction control commands.  These should include every keyword that
@@ -1805,7 +1739,7 @@ command_no_begin(const char *query)
 
 		wordlen = 0;
 		while (isalpha((unsigned char) query[wordlen]))
-			wordlen += PQmblenBounded(&query[wordlen], pset.encoding);
+			wordlen += PQmblen(&query[wordlen], pset.encoding);
 
 		if (wordlen == 11 && pg_strncasecmp(query, "transaction", 11) == 0)
 			return true;
@@ -1839,7 +1773,7 @@ command_no_begin(const char *query)
 
 		wordlen = 0;
 		while (isalpha((unsigned char) query[wordlen]))
-			wordlen += PQmblenBounded(&query[wordlen], pset.encoding);
+			wordlen += PQmblen(&query[wordlen], pset.encoding);
 
 		if (wordlen == 8 && pg_strncasecmp(query, "database", 8) == 0)
 			return true;
@@ -1855,7 +1789,7 @@ command_no_begin(const char *query)
 
 			wordlen = 0;
 			while (isalpha((unsigned char) query[wordlen]))
-				wordlen += PQmblenBounded(&query[wordlen], pset.encoding);
+				wordlen += PQmblen(&query[wordlen], pset.encoding);
 		}
 
 		if (wordlen == 5 && pg_strncasecmp(query, "index", 5) == 0)
@@ -1866,7 +1800,7 @@ command_no_begin(const char *query)
 
 			wordlen = 0;
 			while (isalpha((unsigned char) query[wordlen]))
-				wordlen += PQmblenBounded(&query[wordlen], pset.encoding);
+				wordlen += PQmblen(&query[wordlen], pset.encoding);
 
 			if (wordlen == 12 && pg_strncasecmp(query, "concurrently", 12) == 0)
 				return true;
@@ -1883,7 +1817,7 @@ command_no_begin(const char *query)
 
 		wordlen = 0;
 		while (isalpha((unsigned char) query[wordlen]))
-			wordlen += PQmblenBounded(&query[wordlen], pset.encoding);
+			wordlen += PQmblen(&query[wordlen], pset.encoding);
 
 		/* ALTER SYSTEM isn't allowed in xacts */
 		if (wordlen == 6 && pg_strncasecmp(query, "system", 6) == 0)
@@ -1906,7 +1840,7 @@ command_no_begin(const char *query)
 
 		wordlen = 0;
 		while (isalpha((unsigned char) query[wordlen]))
-			wordlen += PQmblenBounded(&query[wordlen], pset.encoding);
+			wordlen += PQmblen(&query[wordlen], pset.encoding);
 
 		if (wordlen == 8 && pg_strncasecmp(query, "database", 8) == 0)
 			return true;
@@ -1924,7 +1858,7 @@ command_no_begin(const char *query)
 
 			wordlen = 0;
 			while (isalpha((unsigned char) query[wordlen]))
-				wordlen += PQmblenBounded(&query[wordlen], pset.encoding);
+				wordlen += PQmblen(&query[wordlen], pset.encoding);
 
 			if (wordlen == 12 && pg_strncasecmp(query, "concurrently", 12) == 0)
 				return true;
@@ -1944,7 +1878,7 @@ command_no_begin(const char *query)
 
 		wordlen = 0;
 		while (isalpha((unsigned char) query[wordlen]))
-			wordlen += PQmblenBounded(&query[wordlen], pset.encoding);
+			wordlen += PQmblen(&query[wordlen], pset.encoding);
 
 		if (wordlen == 3 && pg_strncasecmp(query, "all", 3) == 0)
 			return true;
@@ -1980,7 +1914,7 @@ is_select_command(const char *query)
 	 */
 	wordlen = 0;
 	while (isalpha((unsigned char) query[wordlen]))
-		wordlen += PQmblenBounded(&query[wordlen], pset.encoding);
+		wordlen += PQmblen(&query[wordlen], pset.encoding);
 
 	if (wordlen == 6 && pg_strncasecmp(query, "select", 6) == 0)
 		return true;
