@@ -1,3 +1,5 @@
+/* Convert timestamp from pg_time_t to struct pg_tm.  */
+
 /*
  * This file is in the public domain, so clarified as of
  * 1996-06-05 by Arthur David Olson.
@@ -26,15 +28,15 @@
 #ifndef WILDABBR
 /*
  * Someone might make incorrect use of a time zone abbreviation:
- *	1.  They might reference tzname[0] before calling tzset (explicitly
+ *	1.	They might reference tzname[0] before calling tzset (explicitly
  *		or implicitly).
- *	2.  They might reference tzname[1] before calling tzset (explicitly
+ *	2.	They might reference tzname[1] before calling tzset (explicitly
  *		or implicitly).
- *	3.  They might reference tzname[1] after setting to a time zone
+ *	3.	They might reference tzname[1] after setting to a time zone
  *		in which Daylight Saving Time is never observed.
- *	4.  They might reference tzname[0] after setting to a time zone
+ *	4.	They might reference tzname[0] after setting to a time zone
  *		in which Standard Time is never observed.
- *	5.  They might reference tm.TM_ZONE after calling offtime.
+ *	5.	They might reference tm.TM_ZONE after calling offtime.
  * What's best to do in the above cases is open to debate;
  * for now, we just set things up so that in any of the five cases
  * WILDABBR is used. Another possibility: initialize tzname[0] to the
@@ -44,31 +46,26 @@
  * that tzname[0] has the "normal" length of three characters).
  */
 #define WILDABBR	"   "
-#endif   /* !defined WILDABBR */
+#endif							/* !defined WILDABBR */
 
 static const char wildabbr[] = WILDABBR;
 
 static const char gmt[] = "GMT";
 
-/* The minimum and maximum finite time values.  This assumes no padding.  */
-static const pg_time_t time_t_min = MINVAL(pg_time_t, TYPE_BIT(pg_time_t));
-static const pg_time_t time_t_max = MAXVAL(pg_time_t, TYPE_BIT(pg_time_t));
-
 /*
- * We cache the result of trying to load the TZDEFRULES zone here.
+ * PG: We cache the result of trying to load the TZDEFRULES zone here.
  * tzdefrules_loaded is 0 if not tried yet, +1 if good, -1 if failed.
  */
-static struct state tzdefrules_s;
+static struct state *tzdefrules_s = NULL;
 static int	tzdefrules_loaded = 0;
 
 /*
  * The DST rules to use if TZ has no rules and we can't load TZDEFRULES.
- * We default to US rules as of 1999-08-17.
- * POSIX 1003.1 section 8.1.1 says that the default DST rules are
- * implementation dependent; for historical reasons, US rules are a
- * common default.
+ * Default to US rules as of 2017-05-07.
+ * POSIX does not specify the default DST rules;
+ * for historical reasons, US rules are a common default.
  */
-#define TZDEFRULESTRING ",M4.1.0,M10.5.0"
+#define TZDEFRULESTRING ",M3.2.0,M11.1.0"
 
 /* structs ttinfo, lsinfo, state have been moved to pgtz.h */
 
@@ -112,7 +109,7 @@ static struct pg_tm tm;
 
 /* Initialize *S to a value based on GMTOFF, ISDST, and ABBRIND.  */
 static void
-init_ttinfo(struct ttinfo * s, int32 gmtoff, bool isdst, int abbrind)
+init_ttinfo(struct ttinfo *s, int32 gmtoff, bool isdst, int abbrind)
 {
 	s->tt_gmtoff = gmtoff;
 	s->tt_isdst = isdst;
@@ -122,7 +119,7 @@ init_ttinfo(struct ttinfo * s, int32 gmtoff, bool isdst, int abbrind)
 }
 
 static int32
-detzcode(const char *codep)
+detzcode(const char *const codep)
 {
 	int32		result;
 	int			i;
@@ -148,7 +145,7 @@ detzcode(const char *codep)
 }
 
 static int64
-detzcode64(const char *codep)
+detzcode64(const char *const codep)
 {
 	uint64		result;
 	int			i;
@@ -189,16 +186,14 @@ union input_buffer
 
 	/* The entire buffer.  */
 	char		buf[2 * sizeof(struct tzhead) + 2 * sizeof(struct state)
-					+			4 * TZ_MAX_TIMES];
+					+ 4 * TZ_MAX_TIMES];
 };
 
 /* Local storage needed for 'tzloadbody'.  */
 union local_storage
 {
-	/* We don't need the "fullname" member */
-
 	/* The results of analyzing the file's contents after it is opened.  */
-	struct
+	struct file_analysis
 	{
 		/* The input buffer.  */
 		union input_buffer u;
@@ -206,6 +201,8 @@ union local_storage
 		/* A temporary state used for parsing a TZ string in the file.  */
 		struct state st;
 	}			u;
+
+	/* We don't need the "fullname" member */
 };
 
 /* Load tz data from the file named NAME into *SP.  Read extended
@@ -215,8 +212,8 @@ union local_storage
  * given name is stored there (the buffer must be > TZ_STRLEN_MAX bytes!).
  */
 static int
-tzloadbody(char const * name, char *canonname, struct state * sp, bool doextend,
-		   union local_storage * lsp)
+tzloadbody(char const *name, char *canonname, struct state *sp, bool doextend,
+		   union local_storage *lsp)
 {
 	int			i;
 	int			fid;
@@ -255,14 +252,21 @@ tzloadbody(char const * name, char *canonname, struct state * sp, bool doextend,
 	{
 		int32		ttisstdcnt = detzcode(up->tzhead.tzh_ttisstdcnt);
 		int32		ttisgmtcnt = detzcode(up->tzhead.tzh_ttisgmtcnt);
+		int64		prevtr = 0;
+		int32		prevcorr = 0;
 		int32		leapcnt = detzcode(up->tzhead.tzh_leapcnt);
 		int32		timecnt = detzcode(up->tzhead.tzh_timecnt);
 		int32		typecnt = detzcode(up->tzhead.tzh_typecnt);
 		int32		charcnt = detzcode(up->tzhead.tzh_charcnt);
 		char const *p = up->buf + tzheadsize;
 
+		/*
+		 * Although tzfile(5) currently requires typecnt to be nonzero,
+		 * support future formats that may allow zero typecnt in files that
+		 * have a TZ string and no transitions.
+		 */
 		if (!(0 <= leapcnt && leapcnt < TZ_MAX_LEAPS
-			  && 0 < typecnt && typecnt < TZ_MAX_TYPES
+			  && 0 <= typecnt && typecnt < TZ_MAX_TYPES
 			  && 0 <= timecnt && timecnt < TZ_MAX_TIMES
 			  && 0 <= charcnt && charcnt < TZ_MAX_CHARS
 			  && (ttisstdcnt == typecnt || ttisstdcnt == 0)
@@ -270,7 +274,7 @@ tzloadbody(char const * name, char *canonname, struct state * sp, bool doextend,
 			return EINVAL;
 		if (nread
 			< (tzheadsize		/* struct tzhead */
-			   + timecnt * stored		/* ats */
+			   + timecnt * stored	/* ats */
 			   + timecnt		/* types */
 			   + typecnt * 6	/* ttinfos */
 			   + charcnt		/* chars */
@@ -285,8 +289,8 @@ tzloadbody(char const * name, char *canonname, struct state * sp, bool doextend,
 
 		/*
 		 * Read transitions, discarding those out of pg_time_t range. But
-		 * pretend the last transition before time_t_min occurred at
-		 * time_t_min.
+		 * pretend the last transition before TIME_T_MIN occurred at
+		 * TIME_T_MIN.
 		 */
 		timecnt = 0;
 		for (i = 0; i < sp->timecnt; ++i)
@@ -294,12 +298,12 @@ tzloadbody(char const * name, char *canonname, struct state * sp, bool doextend,
 			int64		at
 			= stored == 4 ? detzcode(p) : detzcode64(p);
 
-			sp->types[i] = at <= time_t_max;
+			sp->types[i] = at <= TIME_T_MAX;
 			if (sp->types[i])
 			{
 				pg_time_t	attime
-				= ((TYPE_SIGNED(pg_time_t) ? at < time_t_min : at < 0)
-				   ? time_t_min : at);
+				= ((TYPE_SIGNED(pg_time_t) ? at < TIME_T_MIN : at < 0)
+				   ? TIME_T_MIN : at);
 
 				if (timecnt && attime <= sp->ats[timecnt - 1])
 				{
@@ -354,20 +358,22 @@ tzloadbody(char const * name, char *canonname, struct state * sp, bool doextend,
 			int32		corr = detzcode(p + stored);
 
 			p += stored + 4;
-			if (tr <= time_t_max)
+			/* Leap seconds cannot occur before the Epoch.  */
+			if (tr < 0)
+				return EINVAL;
+			if (tr <= TIME_T_MAX)
 			{
-				pg_time_t	trans
-				= ((TYPE_SIGNED(pg_time_t) ? tr < time_t_min : tr < 0)
-				   ? time_t_min : tr);
-
-				if (leapcnt && trans <= sp->lsis[leapcnt - 1].ls_trans)
-				{
-					if (trans < sp->lsis[leapcnt - 1].ls_trans)
-						return EINVAL;
-					leapcnt--;
-				}
-				sp->lsis[leapcnt].ls_trans = trans;
-				sp->lsis[leapcnt].ls_corr = corr;
+				/*
+				 * Leap seconds cannot occur more than once per UTC month, and
+				 * UTC months are at least 28 days long (minus 1 second for a
+				 * negative leap second).  Each leap second's correction must
+				 * differ from the previous one's by 1 second.
+				 */
+				if (tr - prevtr < 28 * SECSPERDAY - 1
+					|| (corr != prevcorr - 1 && corr != prevcorr + 1))
+					return EINVAL;
+				sp->lsis[leapcnt].ls_trans = prevtr = tr;
+				sp->lsis[leapcnt].ls_corr = prevcorr = corr;
 				leapcnt++;
 			}
 		}
@@ -417,8 +423,7 @@ tzloadbody(char const * name, char *canonname, struct state * sp, bool doextend,
 		struct state *ts = &lsp->u.st;
 
 		up->buf[nread - 1] = '\0';
-		if (tzparse(&up->buf[1], ts, false)
-			&& ts->typecnt == 2)
+		if (tzparse(&up->buf[1], ts, false))
 		{
 			/*
 			 * Attempt to reuse existing abbreviations. Without this,
@@ -431,7 +436,7 @@ tzloadbody(char const * name, char *canonname, struct state * sp, bool doextend,
 			int			gotabbr = 0;
 			int			charcnt = sp->charcnt;
 
-			for (i = 0; i < 2; i++)
+			for (i = 0; i < ts->typecnt; i++)
 			{
 				char	   *tsabbr = ts->chars + ts->ttis[i].tt_abbrind;
 				int			j;
@@ -456,7 +461,7 @@ tzloadbody(char const * name, char *canonname, struct state * sp, bool doextend,
 					}
 				}
 			}
-			if (gotabbr == 2)
+			if (gotabbr == ts->typecnt)
 			{
 				sp->charcnt = charcnt;
 
@@ -471,7 +476,8 @@ tzloadbody(char const * name, char *canonname, struct state * sp, bool doextend,
 					sp->timecnt--;
 
 				for (i = 0; i < ts->timecnt; i++)
-					if (sp->ats[sp->timecnt - 1] < ts->ats[i])
+					if (sp->timecnt == 0
+						|| sp->ats[sp->timecnt - 1] < ts->ats[i])
 						break;
 				while (i < ts->timecnt
 					   && sp->timecnt < TZ_MAX_TIMES)
@@ -482,11 +488,13 @@ tzloadbody(char const * name, char *canonname, struct state * sp, bool doextend,
 					sp->timecnt++;
 					i++;
 				}
-				sp->ttis[sp->typecnt++] = ts->ttis[0];
-				sp->ttis[sp->typecnt++] = ts->ttis[1];
+				for (i = 0; i < ts->typecnt; i++)
+					sp->ttis[sp->typecnt++] = ts->ttis[i];
 			}
 		}
 	}
+	if (sp->typecnt == 0)
+		return EINVAL;
 	if (sp->timecnt > 1)
 	{
 		for (i = 1; i < sp->timecnt; ++i)
@@ -508,7 +516,19 @@ tzloadbody(char const * name, char *canonname, struct state * sp, bool doextend,
 	}
 
 	/*
-	 * If type 0 is is unused in transitions, it's the type to use for early
+	 * Infer sp->defaulttype from the data.  Although this default type is
+	 * always zero for data from recent tzdb releases, things are trickier for
+	 * data from tzdb 2018e or earlier.
+	 *
+	 * The first set of heuristics work around bugs in 32-bit data generated
+	 * by tzdb 2013c or earlier.  The workaround is for zones like
+	 * Australia/Macquarie where timestamps before the first transition have a
+	 * time type that is not the earliest standard-time type.  See:
+	 * https://mm.icann.org/pipermail/tz/2013-May/019368.html
+	 */
+
+	/*
+	 * If type 0 is unused in transitions, it's the type to use for early
 	 * times.
 	 */
 	for (i = 0; i < sp->timecnt; ++i)
@@ -529,6 +549,11 @@ tzloadbody(char const * name, char *canonname, struct state * sp, bool doextend,
 				break;
 	}
 
+	/*
+	 * The next heuristics are for data generated by tzdb 2018e or earlier,
+	 * for zones like EST5EDT where the first transition is to DST.
+	 */
+
 	/*
 	 * If no result yet, find the first standard type. If there is none, punt
 	 * to type zero.
@@ -543,7 +568,14 @@ tzloadbody(char const * name, char *canonname, struct state * sp, bool doextend,
 				break;
 			}
 	}
+
+	/*
+	 * A simple 'sp->defaulttype = 0;' would suffice here if we didn't have to
+	 * worry about 2018e-or-earlier data.  Even simpler would be to remove the
+	 * defaulttype member and just use 0 in its place.
+	 */
 	sp->defaulttype = i;
+
 	return 0;
 }
 
@@ -553,7 +585,7 @@ tzloadbody(char const * name, char *canonname, struct state * sp, bool doextend,
  * given name is stored there (the buffer must be > TZ_STRLEN_MAX bytes!).
  */
 int
-tzload(const char *name, char *canonname, struct state * sp, bool doextend)
+tzload(const char *name, char *canonname, struct state *sp, bool doextend)
 {
 	union local_storage *lsp = malloc(sizeof *lsp);
 
@@ -569,7 +601,7 @@ tzload(const char *name, char *canonname, struct state * sp, bool doextend)
 }
 
 static bool
-typesequiv(const struct state * sp, int a, int b)
+typesequiv(const struct state *sp, int a, int b)
 {
 	bool		result;
 
@@ -602,10 +634,11 @@ static const int year_lengths[2] = {
 };
 
 /*
- * Given a pointer into a time zone string, scan until a character that is not
- * a valid character in a zone name is found. Return a pointer to that
- * character.
+ * Given a pointer into a timezone string, scan until a character that is not
+ * a valid character in a time zone abbreviation is found.
+ * Return a pointer to that character.
  */
+
 static const char *
 getzname(const char *strp)
 {
@@ -618,15 +651,17 @@ getzname(const char *strp)
 }
 
 /*
- * Given a pointer into an extended time zone string, scan until the ending
- * delimiter of the zone name is located. Return a pointer to the delimiter.
+ * Given a pointer into an extended timezone string, scan until the ending
+ * delimiter of the time zone abbreviation is located.
+ * Return a pointer to the delimiter.
  *
  * As with getzname above, the legal character set is actually quite
  * restricted, with other characters producing undefined results.
  * We don't do any checking here; checking is done later in common-case code.
  */
+
 static const char *
-getqzname(const char *strp, int delim)
+getqzname(const char *strp, const int delim)
 {
 	int			c;
 
@@ -636,13 +671,14 @@ getqzname(const char *strp, int delim)
 }
 
 /*
- * Given a pointer into a time zone string, extract a number from that string.
+ * Given a pointer into a timezone string, extract a number from that string.
  * Check that the number is within a specified range; if it is not, return
  * NULL.
  * Otherwise, return a pointer to the first character not part of the number.
  */
+
 static const char *
-getnum(const char *strp, int *nump, int min, int max)
+getnum(const char *strp, int *const nump, const int min, const int max)
 {
 	char		c;
 	int			num;
@@ -664,14 +700,15 @@ getnum(const char *strp, int *nump, int min, int max)
 }
 
 /*
- * Given a pointer into a time zone string, extract a number of seconds,
+ * Given a pointer into a timezone string, extract a number of seconds,
  * in hh[:mm[:ss]] form, from the string.
  * If any error occurs, return NULL.
  * Otherwise, return a pointer to the first character not part of the number
  * of seconds.
  */
+
 static const char *
-getsecs(const char *strp, int32 *secsp)
+getsecs(const char *strp, int32 *const secsp)
 {
 	int			num;
 
@@ -705,13 +742,14 @@ getsecs(const char *strp, int32 *secsp)
 }
 
 /*
- * Given a pointer into a time zone string, extract an offset, in
+ * Given a pointer into a timezone string, extract an offset, in
  * [+-]hh[:mm[:ss]] form, from the string.
  * If any error occurs, return NULL.
  * Otherwise, return a pointer to the first character not part of the time.
  */
+
 static const char *
-getoffset(const char *strp, int32 *offsetp)
+getoffset(const char *strp, int32 *const offsetp)
 {
 	bool		neg = false;
 
@@ -731,13 +769,14 @@ getoffset(const char *strp, int32 *offsetp)
 }
 
 /*
- * Given a pointer into a time zone string, extract a rule in the form
+ * Given a pointer into a timezone string, extract a rule in the form
  * date[/time]. See POSIX section 8 for the format of "date" and "time".
  * If a valid rule is not found, return NULL.
  * Otherwise, return a pointer to the first character not part of the rule.
  */
+
 static const char *
-getrule(const char *strp, struct rule * rulep)
+getrule(const char *strp, struct rule *const rulep)
 {
 	if (*strp == 'J')
 	{
@@ -788,7 +827,7 @@ getrule(const char *strp, struct rule * rulep)
 		strp = getoffset(strp, &rulep->r_time);
 	}
 	else
-		rulep->r_time = 2 * SECSPERHOUR;		/* default = 2:00:00 */
+		rulep->r_time = 2 * SECSPERHOUR;	/* default = 2:00:00 */
 	return strp;
 }
 
@@ -796,14 +835,15 @@ getrule(const char *strp, struct rule * rulep)
  * Given a year, a rule, and the offset from UT at the time that rule takes
  * effect, calculate the year-relative time that rule takes effect.
  */
+
 static int32
-transtime(int year, const struct rule * rulep,
-		  int32 offset)
+transtime(const int year, const struct rule *const rulep,
+		  const int32 offset)
 {
 	bool		leapyear;
 	int32		value;
-	int			i,
-				d,
+	int			i;
+	int			d,
 				m1,
 				yy0,
 				yy1,
@@ -894,7 +934,7 @@ transtime(int year, const struct rule * rulep,
  * Returns true on success, false on failure.
  */
 bool
-tzparse(const char *name, struct state * sp, bool lastditch)
+tzparse(const char *name, struct state *sp, bool lastditch)
 {
 	const char *stdname;
 	const char *dstname = NULL;
@@ -909,20 +949,10 @@ tzparse(const char *name, struct state * sp, bool lastditch)
 	stdname = name;
 	if (lastditch)
 	{
-		/*
-		 * This is intentionally somewhat different from the IANA code.  We do
-		 * not want to invoke tzload() in the lastditch case: we can't assume
-		 * pg_open_tzfile() is sane yet, and we don't care about leap seconds
-		 * anyway.
-		 */
+		/* Unlike IANA, don't assume name is exactly "GMT" */
 		stdlen = strlen(name);	/* length of standard zone name */
 		name += stdlen;
-		if (stdlen >= sizeof sp->chars)
-			stdlen = (sizeof sp->chars) - 1;
-		charcnt = stdlen + 1;
 		stdoffset = 0;
-		sp->goback = sp->goahead = false;		/* simulate failed tzload() */
-		load_ok = false;
 	}
 	else
 	{
@@ -946,27 +976,23 @@ tzparse(const char *name, struct state * sp, bool lastditch)
 		name = getoffset(name, &stdoffset);
 		if (name == NULL)
 			return false;
-		charcnt = stdlen + 1;
-		if (sizeof sp->chars < charcnt)
-			return false;
-
-		/*
-		 * This bit also differs from the IANA code, which doesn't make any
-		 * attempt to avoid repetitive loadings of the TZDEFRULES zone.
-		 */
-		if (tzdefrules_loaded == 0)
-		{
-			if (tzload(TZDEFRULES, NULL, &tzdefrules_s, false) == 0)
-				tzdefrules_loaded = 1;
-			else
-				tzdefrules_loaded = -1;
-		}
-		load_ok = (tzdefrules_loaded > 0);
-		if (load_ok)
-			memcpy(sp, &tzdefrules_s, sizeof(struct state));
 	}
-	if (!load_ok)
-		sp->leapcnt = 0;		/* so, we're off a little */
+	charcnt = stdlen + 1;
+	if (sizeof sp->chars < charcnt)
+		return false;
+
+	/*
+	 * The IANA code always tries tzload(TZDEFRULES) here.  We do not want to
+	 * do that; it would be bad news in the lastditch case, where we can't
+	 * assume pg_open_tzfile() is sane yet.  Moreover, the only reason to do
+	 * it unconditionally is to absorb the TZDEFRULES zone's leap second info,
+	 * which we don't want to do anyway.  Without that, we only need to load
+	 * TZDEFRULES if the zone name specifies DST but doesn't incorporate a
+	 * POSIX-style transition date rule, which is not a common case.
+	 */
+	sp->goback = sp->goahead = false;	/* simulate failed tzload() */
+	sp->leapcnt = 0;			/* intentionally assume no leap seconds */
+
 	if (*name != '\0')
 	{
 		if (*name == '<')
@@ -982,7 +1008,7 @@ tzparse(const char *name, struct state * sp, bool lastditch)
 		{
 			dstname = name;
 			name = getzname(name);
-			dstlen = name - dstname;	/* length of DST zone name */
+			dstlen = name - dstname;	/* length of DST abbr. */
 		}
 		if (!dstlen)
 			return false;
@@ -997,8 +1023,38 @@ tzparse(const char *name, struct state * sp, bool lastditch)
 		}
 		else
 			dstoffset = stdoffset - SECSPERHOUR;
-		if (*name == '\0' && !load_ok)
-			name = TZDEFRULESTRING;
+		if (*name == '\0')
+		{
+			/*
+			 * The POSIX zone name does not provide a transition-date rule.
+			 * Here we must load the TZDEFRULES zone, if possible, to serve as
+			 * source data for the transition dates.  Unlike the IANA code, we
+			 * try to cache the data so it's only loaded once.
+			 */
+			if (tzdefrules_loaded == 0)
+			{
+				/* Allocate on first use */
+				if (tzdefrules_s == NULL)
+					tzdefrules_s = (struct state *) malloc(sizeof(struct state));
+				if (tzdefrules_s != NULL)
+				{
+					if (tzload(TZDEFRULES, NULL, tzdefrules_s, false) == 0)
+						tzdefrules_loaded = 1;
+					else
+						tzdefrules_loaded = -1;
+					/* In any case, we ignore leap-second data from the file */
+					tzdefrules_s->leapcnt = 0;
+				}
+			}
+			load_ok = (tzdefrules_loaded > 0);
+			if (load_ok)
+				memcpy(sp, tzdefrules_s, sizeof(struct state));
+			else
+			{
+				/* If we can't load TZDEFRULES, fall back to hard-wired rule */
+				name = TZDEFRULESTRING;
+			}
+		}
 		if (*name == ',' || *name == ';')
 		{
 			struct rule start;
@@ -1024,8 +1080,8 @@ tzparse(const char *name, struct state * sp, bool lastditch)
 			/*
 			 * Two transitions per year, from EPOCH_YEAR forward.
 			 */
-			init_ttinfo(&sp->ttis[0], -dstoffset, true, stdlen + 1);
-			init_ttinfo(&sp->ttis[1], -stdoffset, false, 0);
+			init_ttinfo(&sp->ttis[0], -stdoffset, false, 0);
+			init_ttinfo(&sp->ttis[1], -dstoffset, true, stdlen + 1);
 			sp->defaulttype = 0;
 			timecnt = 0;
 			janfirst = 0;
@@ -1074,19 +1130,15 @@ tzparse(const char *name, struct state * sp, bool lastditch)
 					if (!increment_overflow_time
 						(&sp->ats[timecnt],
 						 janoffset + starttime))
-						sp->types[timecnt++] = reversed;
-					else if (janoffset)
-						sp->defaulttype = reversed;
+						sp->types[timecnt++] = !reversed;
 					sp->ats[timecnt] = janfirst;
 					if (!increment_overflow_time
 						(&sp->ats[timecnt],
 						 janoffset + endtime))
 					{
-						sp->types[timecnt++] = !reversed;
+						sp->types[timecnt++] = reversed;
 						yearlim = year + YEARSPERREPEAT + 1;
 					}
-					else if (janoffset)
-						sp->defaulttype = !reversed;
 				}
 				if (increment_overflow_time
 					(&janfirst, janoffset + yearsecs))
@@ -1095,7 +1147,10 @@ tzparse(const char *name, struct state * sp, bool lastditch)
 			}
 			sp->timecnt = timecnt;
 			if (!timecnt)
+			{
+				sp->ttis[0] = sp->ttis[1];
 				sp->typecnt = 1;	/* Perpetual DST.  */
+			}
 			else if (YEARSPERREPEAT < year - yearbeg)
 				sp->goback = sp->goahead = true;
 		}
@@ -1158,12 +1213,12 @@ tzparse(const char *name, struct state * sp, bool lastditch)
 				else
 				{
 					/*
-					 * If summer time is in effect, and the transition time
-					 * was not specified as standard time, add the summer time
-					 * offset to the transition time; otherwise, add the
-					 * standard time offset to the transition time.
+					 * If daylight saving time is in effect, and the
+					 * transition time was not specified as standard time, add
+					 * the daylight saving time offset to the transition time;
+					 * otherwise, add the standard time offset to the
+					 * transition time.
 					 */
-
 					/*
 					 * Transitions from DST to DDST will effectively disappear
 					 * since POSIX provides for only one DST offset.
@@ -1217,7 +1272,7 @@ tzparse(const char *name, struct state * sp, bool lastditch)
 }
 
 static void
-gmtload(struct state * sp)
+gmtload(struct state *const sp)
 {
 	if (tzload(gmt, NULL, sp, true) != 0)
 		tzparse(gmt, sp, true);
@@ -1231,8 +1286,8 @@ gmtload(struct state * sp)
  * but it *is* desirable.)
  */
 static struct pg_tm *
-localsub(struct state const * sp, pg_time_t const * timep,
-		 struct pg_tm * tmp)
+localsub(struct state const *sp, pg_time_t const *timep,
+		 struct pg_tm *const tmp)
 {
 	const struct ttinfo *ttisp;
 	int			i;
@@ -1300,6 +1355,11 @@ localsub(struct state const * sp, pg_time_t const * timep,
 	}
 	ttisp = &sp->ttis[i];
 
+	/*
+	 * To get (wrong) behavior that's compatible with System V Release 2.0
+	 * you'd replace the statement below with t += ttisp->tt_gmtoff;
+	 * timesub(&t, 0L, sp, tmp);
+	 */
 	result = timesub(&t, ttisp->tt_gmtoff, sp, tmp);
 	if (result)
 	{
@@ -1322,21 +1382,25 @@ pg_localtime(const pg_time_t *timep, const pg_tz *tz)
  *
  * Except we have a private "struct state" for GMT, so no sp is passed in.
  */
+
 static struct pg_tm *
-gmtsub(pg_time_t const * timep, int32 offset, struct pg_tm * tmp)
+gmtsub(pg_time_t const *timep, int32 offset,
+	   struct pg_tm *tmp)
 {
 	struct pg_tm *result;
 
 	/* GMT timezone state data is kept here */
-	static struct state gmtmem;
-	static bool gmt_is_set = false;
-#define gmtptr		(&gmtmem)
+	static struct state *gmtptr = NULL;
 
-	if (!gmt_is_set)
+	if (gmtptr == NULL)
 	{
-		gmt_is_set = true;
+		/* Allocate on first use */
+		gmtptr = (struct state *) malloc(sizeof(struct state));
+		if (gmtptr == NULL)
+			return NULL;		/* errno should be set by malloc */
 		gmtload(gmtptr);
 	}
+
 	result = timesub(timep, offset, gmtptr, tmp);
 
 	/*
@@ -1361,16 +1425,24 @@ pg_gmtime(const pg_time_t *timep)
  * Return the number of leap years through the end of the given year
  * where, to make the math easy, the answer for year zero is defined as zero.
  */
+
+static int
+leaps_thru_end_of_nonneg(int y)
+{
+	return y / 4 - y / 100 + y / 400;
+}
+
 static int
 leaps_thru_end_of(const int y)
 {
-	return (y >= 0) ? (y / 4 - y / 100 + y / 400) :
-		-(leaps_thru_end_of(-(y + 1)) + 1);
+	return (y < 0
+			? -1 - leaps_thru_end_of_nonneg(-1 - y)
+			: leaps_thru_end_of_nonneg(y));
 }
 
 static struct pg_tm *
 timesub(const pg_time_t *timep, int32 offset,
-		const struct state * sp, struct pg_tm * tmp)
+		const struct state *sp, struct pg_tm *tmp)
 {
 	const struct lsinfo *lp;
 	pg_time_t	tdays;
@@ -1390,22 +1462,9 @@ timesub(const pg_time_t *timep, int32 offset,
 		lp = &sp->lsis[i];
 		if (*timep >= lp->ls_trans)
 		{
-			if (*timep == lp->ls_trans)
-			{
-				hit = ((i == 0 && lp->ls_corr > 0) ||
-					   lp->ls_corr > sp->lsis[i - 1].ls_corr);
-				if (hit)
-					while (i > 0 &&
-						   sp->lsis[i].ls_trans ==
-						   sp->lsis[i - 1].ls_trans + 1 &&
-						   sp->lsis[i].ls_corr ==
-						   sp->lsis[i - 1].ls_corr + 1)
-					{
-						++hit;
-						--i;
-					}
-			}
 			corr = lp->ls_corr;
+			hit = (*timep == lp->ls_trans
+				   && (i == 0 ? 0 : lp[-1].ls_corr) < corr);
 			break;
 		}
 	}
@@ -1529,13 +1588,13 @@ increment_overflow_time(pg_time_t *tp, int32 j)
 {
 	/*----------
 	 * This is like
-	 * 'if (! (time_t_min <= *tp + j && *tp + j <= time_t_max)) ...',
+	 * 'if (! (TIME_T_MIN <= *tp + j && *tp + j <= TIME_T_MAX)) ...',
 	 * except that it does the right thing even if *tp + j would overflow.
 	 *----------
 	 */
 	if (!(j < 0
-		  ? (TYPE_SIGNED(pg_time_t) ? time_t_min - j <= *tp : -1 - j < *tp)
-		  : *tp <= time_t_max - j))
+		  ? (TYPE_SIGNED(pg_time_t) ? TIME_T_MIN - j <= *tp : -1 - j < *tp)
+		  : *tp <= TIME_T_MAX - j))
 		return true;
 	*tp += j;
 	return false;
